@@ -22,8 +22,6 @@ new local scope in the language definition : Module, ClassDef, FunctionDef (and
 Lambda, GeneratorExp, DictComp and SetComp to some extent).
 """
 
-from __future__ import print_function
-
 import io
 import itertools
 import warnings
@@ -109,7 +107,7 @@ def function_to_method(n, klass):
 
 def std_special_attributes(self, name, add_locals=True):
     if add_locals:
-        locals = self.locals
+        locals = self._locals
     else:
         locals = {}
     if name == '__name__':
@@ -131,7 +129,7 @@ def builtin_lookup(name):
     if name == '__dict__':
         return builtin_astroid, ()
     try:
-        stmts = builtin_astroid.locals[name]
+        stmts = builtin_astroid._locals[name]
     except KeyError:
         stmts = ()
     return builtin_astroid, stmts
@@ -148,6 +146,18 @@ class LocalsDictNodeNG(node_classes.LookupMixIn, bases.NodeNG):
 
     # dictionary of locals with name as key and node defining the local as
     # value
+    @property
+    def locals(self):
+        util.attribute_to_function_warning('locals', 2.0, 'get_locals')
+        return self._locals
+    @locals.setter
+    def locals(self, _locals):
+        util.attribute_to_function_warning('locals', 2.0, 'get_locals')
+        self._locals = _locals
+    @locals.deleter
+    def locals(self):
+        util.attribute_to_function_warning('locals', 2.0, 'get_locals')
+        del self._locals
 
     def qname(self):
         """return the 'qualified' name of the node, eg module.name,
@@ -171,7 +181,7 @@ class LocalsDictNodeNG(node_classes.LookupMixIn, bases.NodeNG):
     def _scope_lookup(self, node, name, offset=0):
         """XXX method for interfacing the scope lookup"""
         try:
-            stmts = node._filter_stmts(self.locals[name], self, offset)
+            stmts = node._filter_stmts(self._locals[name], self, offset)
         except KeyError:
             stmts = ()
         if stmts:
@@ -192,8 +202,8 @@ class LocalsDictNodeNG(node_classes.LookupMixIn, bases.NodeNG):
 
         if the name is already defined, ignore it
         """
-        #assert not stmt in self.locals.get(name, ()), (self, stmt)
-        self.locals.setdefault(name, []).append(stmt)
+        #assert not stmt in self._locals.get(name, ()), (self, stmt)
+        self._locals.setdefault(name, []).append(stmt)
 
     __setitem__ = set_local
 
@@ -217,7 +227,7 @@ class LocalsDictNodeNG(node_classes.LookupMixIn, bases.NodeNG):
         :param item: the name of the locally defined object
         :raises KeyError: if the name is not defined
         """
-        return self.locals[item][0]
+        return self._locals[item][0]
 
     def __iter__(self):
         """method from the `dict` interface returning an iterator on
@@ -229,7 +239,7 @@ class LocalsDictNodeNG(node_classes.LookupMixIn, bases.NodeNG):
         """method from the `dict` interface returning a tuple containing
         locally defined names
         """
-        return list(self.locals.keys())
+        return list(self._locals.keys())
 
     def values(self):
         """method from the `dict` interface returning a tuple containing
@@ -245,7 +255,7 @@ class LocalsDictNodeNG(node_classes.LookupMixIn, bases.NodeNG):
         return list(zip(self.keys(), self.values()))
 
     def __contains__(self, name):
-        return name in self.locals
+        return name in self._locals
 
 
 class Module(LocalsDictNodeNG):
@@ -258,9 +268,9 @@ class Module(LocalsDictNodeNG):
 
     # the file from which as been extracted the astroid representation. It may
     # be None if the representation has been built from a built-in module
-    file = None
+    source_file = None
     # Alternatively, if built from a string/bytes, this can be set
-    file_bytes = None
+    source_code = None
     # encoding of python source file, so we can get unicode out of it (python2
     # only)
     file_encoding = None
@@ -272,10 +282,10 @@ class Module(LocalsDictNodeNG):
     package = None
     # dictionary of globals with name as key and node defining the global
     # as value
-    globals = None
+    _globals = None
 
     # Future imports
-    future_imports = None
+    _future_imports = None
 
     # names of python special attributes (handled by getattr impl.)
     special_attributes = set(('__name__', '__doc__', '__file__', '__path__',
@@ -283,31 +293,85 @@ class Module(LocalsDictNodeNG):
     # names of module attributes available through the global scope
     scope_attrs = set(('__name__', '__doc__', '__file__', '__path__'))
 
-    _other_fields = ('name', 'doc', 'file', 'path', 'package',
-                     'pure_python', 'future_imports')
-    _other_other_fields = ('locals', 'globals')
-
-    def __init__(self, name, doc, file=None, path=None, package=None,
-                 parent=None, pure_python=True):
+    def __init__(self, name, doc, pure_python=True):
         self.name = name
         self.doc = doc
-        self.file = file
-        self.path = path
-        self.package = package
-        self.parent = parent
         self.pure_python = pure_python
-        self.locals = self.globals = {}
+        self._locals = self._globals = {}
         self.body = []
-        self.future_imports = set()
+        self._future_imports = set()
 
-    def postinit(self, body=None):
-        self.body = body
+    # Future deprecation warnings
+    @property
+    def file(self):
+        util.rename_warning('file', 2.0, 'source_file')
+        return self.source_file
+    @file.setter
+    def file(self, source_file):
+        util.rename_warning('file', 2.0, 'source_file')
+        self.source_file = source_file
+    @file.deleter
+    def file(self):
+        util.rename_warning('file', 2.0, 'source_file')
+        del self.source_file
+
+    @property
+    def path(self):
+        util.rename_warning('path', 2.0, 'source_file')
+        return self.source_file
+    @path.setter
+    def path(self, source_file):
+        util.rename_warning('path', 2.0, 'source_file')
+        self.source_file = source_file
+    @path.deleter
+    def path(self):
+        util.rename_warning('path', 2.0, 'source_file')
+        del self.source_file
+
+    @property
+    def file_bytes(self):
+        util.rename_warning('file_bytes', 2.0, 'source_code')
+        return self.source_code
+    @file_bytes.setter
+    def file_bytes(self, source_code):
+        util.rename_warning('file_bytes', 2.0, 'source_code')
+        self.source_code = source_code
+    @file_bytes.deleter
+    def file_bytes(self):
+        util.rename_warning('file_bytes', 2.0, 'source_code')
+        del self.source_code
+
+    @property
+    def globals(self):
+        util.attribute_to_function_warning('globals', 2.0, 'get_locals')
+        return self._locals
+    @globals.setter
+    def globals(self, _globals):
+        util.attribute_to_function_warning('globals', 2.0, 'get_locals')
+        self._locals = _globals
+    @globals.deleter
+    def globals(self):
+        util.attribute_to_function_warning('globals', 2.0, 'get_locals')
+        del self._locals
+
+    @property
+    def future_imports(self):
+        util.attribute_to_function_warning('future_imports', 2.0, 'future_imports')
+        return self._future_imports
+    @future_imports.setter
+    def future_imports(self, _future_imports):
+        util.attribute_to_function_warning('future_imports', 2.0, 'future_imports')
+        self._future_imports = _future_imports
+    @future_imports.deleter
+    def future_imports(self):
+        util.attribute_to_function_warning('future_imports', 2.0, 'future_imports')
+        del self._future_imports
 
     def _get_stream(self):
-        if self.file_bytes is not None:
-            return io.BytesIO(self.file_bytes)
-        if self.file is not None:
-            stream = open(self.file, 'rb')
+        if self.source_code is not None:
+            return io.BytesIO(self.source_code)
+        if self.source_file is not None:
+            stream = open(self.source_file, 'rb')
             return stream
         return None
 
@@ -342,7 +406,7 @@ class Module(LocalsDictNodeNG):
         return self.fromlineno, self.tolineno
 
     def scope_lookup(self, node, name, offset=0):
-        if name in self.scope_attrs and name not in self.locals:
+        if name in self.scope_attrs and name not in self._locals:
             try:
                 return self, self.getattr(name)
             except exceptions.NotFoundError:
@@ -359,17 +423,19 @@ class Module(LocalsDictNodeNG):
     def getattr(self, name, context=None, ignore_locals=False):
         if name in self.special_attributes:
             if name == '__file__':
-                return [node_classes.const_factory(self.file)] + self.locals.get(name, [])
+                return [node_classes.const_factory(self.source_file)] + self._locals.get(name, [])
             if name == '__path__' and self.package:
-                return [node_classes.List()] + self.locals.get(name, [])
+                return [node_classes.List()] + self._locals.get(name, [])
             return std_special_attributes(self, name)
-        if not ignore_locals and name in self.locals:
-            return self.locals[name]
+        if not ignore_locals and name in self._locals:
+            return self._locals[name]
         if self.package:
             try:
                 return [self.import_module(name, relative_only=True)]
-            except (exceptions.AstroidBuildingException, SyntaxError):
-                util.reraise(exceptions.NotFoundError(name))
+            except exceptions.AstroidBuildingException:
+                raise exceptions.NotFoundError(name)
+            except SyntaxError:
+                raise exceptions.NotFoundError(name)
         raise exceptions.NotFoundError(name)
 
     def igetattr(self, name, context=None):
@@ -382,13 +448,13 @@ class Module(LocalsDictNodeNG):
             return bases._infer_stmts(self.getattr(name, context),
                                       context, frame=self)
         except exceptions.NotFoundError:
-            util.reraise(exceptions.InferenceError(name))
+            raise exceptions.InferenceError(name)
 
     def fully_defined(self):
         """return True if this module has been built from a .py file
         and so contains a complete representation including the code
         """
-        return self.file is not None and self.file.endswith('.py')
+        return self.source_file is not None and self.source_file.endswith('.py')
 
     def statement(self):
         """return the first parent node marked as statement node
@@ -407,7 +473,7 @@ class Module(LocalsDictNodeNG):
     if six.PY2:
         @decorators_mod.cachedproperty
         def _absolute_import_activated(self):
-            for stmt in self.locals.get('absolute_import', ()):
+            for stmt in self._locals.get('absolute_import', ()):
                 if isinstance(stmt, node_classes.ImportFrom) and stmt.modname == '__future__':
                     return True
             return False
@@ -469,6 +535,7 @@ class Module(LocalsDictNodeNG):
             all = self['__all__']
         except KeyError:
             return default
+
         try:
             explicit = next(all.assigned_stmts())
         except exceptions.InferenceError:
@@ -514,67 +581,30 @@ class ComprehensionScope(LocalsDictNodeNG):
 
 class GeneratorExp(ComprehensionScope):
     _astroid_fields = ('elt', 'generators')
-    _other_other_fields = ('locals',)
-    elt = None
-    generators = None
 
-    def __init__(self, lineno=None, col_offset=None, parent=None):
-        self.locals = {}
-        super(GeneratorExp, self).__init__(lineno, col_offset, parent)
-
-    def postinit(self, elt=None, generators=None):
-        self.elt = elt
-        if generators is None:
-            self.generators = []
-        else:
-            self.generators = generators
-
-    def bool_value(self):
-        return True
+    def __init__(self):
+        self._locals = {}
+        self.elt = None
+        self.generators = []
 
 
 class DictComp(ComprehensionScope):
     _astroid_fields = ('key', 'value', 'generators')
-    _other_other_fields = ('locals',)
-    key = None
-    value = None
-    generators = None
 
-    def __init__(self, lineno=None, col_offset=None, parent=None):
-        self.locals = {}
-        super(DictComp, self).__init__(lineno, col_offset, parent)
-
-    def postinit(self, key=None, value=None, generators=None):
-        self.key = key
-        self.value = value
-        if generators is None:
-            self.generators = []
-        else:
-            self.generators = generators
-
-    def bool_value(self):
-        return util.YES
+    def __init__(self):
+        self._locals = {}
+        self.key = None
+        self.value = None
+        self.generators = []
 
 
 class SetComp(ComprehensionScope):
     _astroid_fields = ('elt', 'generators')
-    _other_other_fields = ('locals',)
-    elt = None
-    generators = None
 
-    def __init__(self, lineno=None, col_offset=None, parent=None):
-        self.locals = {}
-        super(SetComp, self).__init__(lineno, col_offset, parent)
-
-    def postinit(self, elt=None, generators=None):
-        self.elt = elt
-        if generators is None:
-            self.generators = []
-        else:
-            self.generators = generators
-
-    def bool_value(self):
-        return util.YES
+    def __init__(self):
+        self._locals = {}
+        self.elt = None
+        self.generators = []
 
 
 class _ListComp(bases.NodeNG):
@@ -583,22 +613,12 @@ class _ListComp(bases.NodeNG):
     elt = None
     generators = None
 
-    def postinit(self, elt=None, generators=None):
-        self.elt = elt
-        self.generators = generators
-
-    def bool_value(self):
-        return util.YES
-
 
 if six.PY3:
     class ListComp(_ListComp, ComprehensionScope):
         """class representing a ListComp node"""
-        _other_other_fields = ('locals',)
-
-        def __init__(self, lineno=None, col_offset=None, parent=None):
-            self.locals = {}
-            super(ListComp, self).__init__(lineno, col_offset, parent)
+        def __init__(self):
+            self._locals = {}
 else:
     class ListComp(_ListComp):
         """class representing a ListComp node"""
@@ -630,21 +650,15 @@ def _infer_decorator_callchain(node):
 
 class Lambda(mixins.FilterStmtsMixin, LocalsDictNodeNG):
     _astroid_fields = ('args', 'body',)
-    _other_other_fields = ('locals',)
     name = '<lambda>'
 
     # function's type, 'function' | 'method' | 'staticmethod' | 'classmethod'
     type = 'function'
 
-    def __init__(self, lineno=None, col_offset=None, parent=None):
-        self.locals = {}
+    def __init__(self):
+        self._locals = {}
         self.args = []
         self.body = []
-        super(Lambda, self).__init__(lineno, col_offset, parent)
-
-    def postinit(self, args, body):
-        self.args = args
-        self.body = body
 
     def pytype(self):
         if 'method' in self.type:
@@ -686,8 +700,6 @@ class Lambda(mixins.FilterStmtsMixin, LocalsDictNodeNG):
             frame = self
         return frame._scope_lookup(node, name, offset)
 
-    def bool_value(self):
-        return True
 
 
 class FunctionDef(bases.Statement, Lambda):
@@ -696,30 +708,32 @@ class FunctionDef(bases.Statement, Lambda):
         returns = None
     else:
         _astroid_fields = ('decorators', 'args', 'body')
-    decorators = None
+
     special_attributes = set(('__name__', '__doc__', '__dict__'))
     is_function = True
     # attributes below are set by the builder module or by raw factories
-    _other_fields = ('name', 'doc')
-    _other_other_fields = ('locals', '_type')
-    _type = None
+    decorators = None
 
-    def __init__(self, name=None, doc=None, lineno=None,
-                 col_offset=None, parent=None):
+    def __init__(self, name, doc):
+        self._locals = {}
+        self.args = []
+        self.body = []
         self.name = name
         self.doc = doc
-        self.instance_attrs = {}
-        super(FunctionDef, self).__init__(lineno, col_offset, parent)
-        if parent:
-            frame = parent.frame()
-            frame.set_local(name, self)
+        self._instance_attrs = {}
 
-    # pylint: disable=arguments-differ; different than Lambdas
-    def postinit(self, args, body, decorators=None, returns=None):
-        self.args = args
-        self.body = body
-        self.decorators = decorators
-        self.returns = returns
+    @property
+    def instance_attrs(self):
+        util.attribute_to_function_warning('instance_attrs', 2.0, 'get_attributes')
+        return self._instance_attrs
+    @instance_attrs.setter
+    def instance_attrs(self, _instance_attrs):
+        util.attribute_to_function_warning('instance_attrs', 2.0, 'get_attributes')
+        self._instance_attrs = _instance_attrs
+    @instance_attrs.deleter
+    def instance_attrs(self):
+        util.attribute_to_function_warning('instance_attrs', 2.0, 'get_attributes')
+        del self._instance_attrs
 
     @decorators_mod.cachedproperty
     def extra_decorators(self):
@@ -845,8 +859,8 @@ class FunctionDef(bases.Statement, Lambda):
         """
         if name == '__module__':
             return [node_classes.const_factory(self.root().qname())]
-        if name in self.instance_attrs:
-            return self.instance_attrs[name]
+        if name in self._instance_attrs:
+            return self._instance_attrs[name]
         return std_special_attributes(self, name, False)
 
     def igetattr(self, name, context=None):
@@ -855,7 +869,7 @@ class FunctionDef(bases.Statement, Lambda):
             return bases._infer_stmts(self.getattr(name, context),
                                       context, frame=self)
         except exceptions.NotFoundError:
-            util.reraise(exceptions.InferenceError(name))
+            raise exceptions.InferenceError(name)
 
     def is_method(self):
         """return true if the function node should be considered as a method"""
@@ -951,9 +965,6 @@ class FunctionDef(bases.Statement, Lambda):
                 except exceptions.InferenceError:
                     yield util.YES
 
-    def bool_value(self):
-        return True
-
 
 class AsyncFunctionDef(FunctionDef):
     """Asynchronous function created with the `async` keyword."""
@@ -1039,23 +1050,6 @@ def _class_type(klass, ancestors=None):
     return klass._type
 
 
-def get_wrapping_class(node):
-    """Obtain the class that *wraps* this node
-
-    We consider that a class wraps a node if the class
-    is a parent for the said node.
-    """
-
-    klass = node.frame()
-    while klass is not None and not isinstance(klass, ClassDef):
-        if klass.parent is None:
-            klass = None
-        else:
-            klass = klass.parent.frame()
-    return klass
-
-
-
 class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG, bases.Statement):
 
     # some of the attributes below are set by the builder module or
@@ -1067,36 +1061,34 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG, bases.Statement):
     decorators = None
     special_attributes = set(('__name__', '__doc__', '__dict__', '__module__',
                               '__bases__', '__mro__', '__subclasses__'))
+
     _type = None
     _metaclass_hack = False
     hide = False
     type = property(_class_type,
                     doc="class'type, possible values are 'class' | "
                     "'metaclass' | 'exception'")
-    _other_fields = ('name', 'doc')
-    _other_other_fields = ('locals', '_newstyle')
-    _newstyle = None
 
-    def __init__(self, name=None, doc=None, lineno=None,
-                 col_offset=None, parent=None):
-        self.instance_attrs = {}
-        self.locals = {}
+    def __init__(self, name, doc):
+        self._instance_attrs = {}
+        self._locals = {}
         self.bases = []
         self.body = []
         self.name = name
         self.doc = doc
-        super(ClassDef, self).__init__(lineno, col_offset, parent)
-        if parent is not None:
-            parent.frame().set_local(name, self)
 
-    def postinit(self, bases, body, decorators, newstyle=None, metaclass=None):
-        self.bases = bases
-        self.body = body
-        self.decorators = decorators
-        if newstyle is not None:
-            self._newstyle = newstyle
-        if metaclass is not None:
-            self._metaclass = metaclass
+    @property
+    def instance_attrs(self):
+        util.attribute_to_function_warning('instance_attrs', 2.0, 'get_attributes')
+        return self._instance_attrs
+    @instance_attrs.setter
+    def instance_attrs(self, _instance_attrs):
+        util.attribute_to_function_warning('instance_attrs', 2.0, 'get_attributes')
+        self._instance_attrs = _instance_attrs
+    @instance_attrs.deleter
+    def instance_attrs(self):
+        util.attribute_to_function_warning('instance_attrs', 2.0, 'get_attributes')
+        del self._instance_attrs
 
     def _newstyle_impl(self, context=None):
         if context is None:
@@ -1107,7 +1099,7 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG, bases.Statement):
             if base._newstyle_impl(context):
                 self._newstyle = True
                 break
-        klass = self.declared_metaclass()
+        klass = self._explicit_metaclass()
         # could be any callable, we'd need to infer the result of klass(name,
         # bases, dict).  punt if it's not a class node.
         if klass is not None and isinstance(klass, ClassDef):
@@ -1183,7 +1175,7 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG, bases.Statement):
             for attr, value in members.items:
                 if (isinstance(attr, node_classes.Const) and
                         isinstance(attr.value, six.string_types)):
-                    result.locals[attr.value] = [value]
+                    result._locals[attr.value] = [value]
 
         result.parent = caller.parent
         return result
@@ -1198,6 +1190,7 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG, bases.Statement):
             yield bases.Instance(self)
 
     def scope_lookup(self, node, name, offset=0):
+        # pylint: disable=redefined-variable-type
         if any(node == base or base.parent_of(node)
                for base in self.bases):
             # Handle the case where we have either a name
@@ -1295,7 +1288,7 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG, bases.Statement):
         which have <name> defined in their instance attribute dictionary
         """
         for astroid in self.ancestors(context=context):
-            if name in astroid.instance_attrs:
+            if name in astroid._instance_attrs:
                 yield astroid
 
     def has_base(self, node):
@@ -1311,10 +1304,10 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG, bases.Statement):
           its parent classes
         """
         try:
-            return self.locals[name]
+            return self._locals[name]
         except KeyError:
             for class_node in self.local_attr_ancestors(name, context):
-                return class_node.locals[name]
+                return class_node._locals[name]
         raise exceptions.NotFoundError(name)
 
     @remove_nodes(node_classes.DelAttr)
@@ -1326,101 +1319,55 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG, bases.Statement):
           if no attribute with this name has been find in this class or
           its parent classes
         """
-        # Return a copy, so we don't modify self.instance_attrs,
+        # Return a copy, so we don't modify self._instance_attrs,
         # which could lead to infinite loop.
-        values = list(self.instance_attrs.get(name, []))
+        values = list(self._instance_attrs.get(name, []))
         # get all values from parents
         for class_node in self.instance_attr_ancestors(name, context):
-            values += class_node.instance_attrs[name]
+            values += class_node._instance_attrs[name]
         if not values:
             raise exceptions.NotFoundError(name)
         return values
 
-    def instanciate_class(self):
+    def instantiate_class(self):
         """return Instance of ClassDef node, else return self"""
         return bases.Instance(self)
 
-    def getattr(self, name, context=None, class_context=True):
-        """Get an attribute from this class, using Python's attribute semantic
+    def instanciate_class(self):
+        """return Instance of ClassDef node, else return self"""
+        util.rename_warning('instanciate_class()', 2.0, 'instantiate_class()')
+        return self.instantiate_class()
 
-        This method doesn't look in the instance_attrs dictionary
-        since it's done by an Instance proxy at inference time.  It
-        may return a YES object if the attribute has not been actually
-        found but a __getattr__ or __getattribute__ method is defined.
-        If *class_context* is given, then it's considered that the
-        attribute is accessed from a class context,
-        e.g. ClassDef.attribute, otherwise it might have been accessed
-        from an instance as well.  If *class_context* is used in that
-        case, then a lookup in the implicit metaclass and the explicit
-        metaclass will be done.
+    def getattr(self, name, context=None):
+        """this method doesn't look in the instance_attrs dictionary since it's
+        done by an Instance proxy at inference time.
 
+        It may return a YES object if the attribute has not been actually
+        found but a __getattr__ or __getattribute__ method is defined
         """
-        values = self.locals.get(name, [])
+        values = self._locals.get(name, [])
         if name in self.special_attributes:
             if name == '__module__':
                 return [node_classes.const_factory(self.root().qname())] + values
-            # FIXME: do we really need the actual list of ancestors?
-            # returning [Tuple()] + values don't break any test
-            # this is ticket http://www.logilab.org/ticket/52785
-            # XXX need proper meta class handling + MRO implementation
-            if name == '__bases__' or (name == '__mro__' and self.newstyle):
+            if name == '__bases__':
                 node = node_classes.Tuple()
-                node.items = self.ancestors(recurs=True, context=context)
+                elts = list(self._inferred_bases(context))
+                node.elts = elts
                 return [node] + values
+            if name == '__mro__' and self.newstyle:
+                mro = self.mro()
+                node = node_classes.Tuple()
+                node.elts = mro
+                return [node]
             return std_special_attributes(self, name)
-        # don't modify the list in self.locals!
+        # don't modify the list in self._locals!
         values = list(values)
         for classnode in self.ancestors(recurs=True, context=context):
-            values += classnode.locals.get(name, [])
-
-        if class_context:
-            values += self._metaclass_lookup_attribute(name, context)
+            values += classnode._locals.get(name, [])
         if not values:
             raise exceptions.NotFoundError(name)
         return values
 
-    def _metaclass_lookup_attribute(self, name, context):
-        """Search the given name in the implicit and the explicit metaclass."""
-        attrs = set()
-        implicit_meta = self.implicit_metaclass()
-        metaclass = self.metaclass()
-        for cls in {implicit_meta, metaclass}:
-            if cls and cls != self and isinstance(cls, ClassDef):
-                cls_attributes = self._get_attribute_from_metaclass(
-                    cls, name, context)
-                attrs.update(set(cls_attributes))
-        return attrs
-
-    def _get_attribute_from_metaclass(self, cls, name, context):
-        try:
-            attrs = cls.getattr(name, context=context,
-                                class_context=True)
-        except exceptions.NotFoundError:
-            return
-
-        for attr in bases._infer_stmts(attrs, context, frame=cls):
-            if not isinstance(attr, FunctionDef):
-                yield attr
-                continue
-
-            if bases._is_property(attr):
-                # TODO(cpopa): don't use a private API.
-                for inferred in attr.infer_call_result(self, context):
-                    yield inferred
-                continue
-            if attr.type == 'classmethod':
-                # If the method is a classmethod, then it will
-                # be bound to the metaclass, not to the class
-                # from where the attribute is retrieved.
-                # get_wrapping_class could return None, so just
-                # default to the current class.
-                frame = get_wrapping_class(attr) or self
-                yield bases.BoundMethod(attr, frame)
-            elif attr.type == 'staticmethod':
-                yield attr
-            else:
-                yield bases.BoundMethod(attr, self)
-
     def igetattr(self, name, context=None):
         """inferred getattr, need special treatment in class to handle
         descriptors
@@ -1448,7 +1395,7 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG, bases.Statement):
                 # class handle some dynamic attributes, return a YES object
                 yield util.YES
             else:
-                util.reraise(exceptions.InferenceError(name))
+                raise exceptions.InferenceError(name)
 
     def has_dynamic_getattr(self, context=None):
         """
@@ -1504,10 +1451,11 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG, bases.Statement):
             return builtin_lookup('type')[1][0]
 
     _metaclass = None
-    def declared_metaclass(self):
-        """Return the explicit declared metaclass for the current class.
+    def _explicit_metaclass(self):
+        """ Return the explicit defined metaclass
+        for the current class.
 
-        An explicit declared metaclass is defined
+        An explicit defined metaclass is defined
         either by passing the ``metaclass`` keyword argument
         in the class definition line (Python 3) or (Python 2) by
         having a ``__metaclass__`` class attribute, or if there are
@@ -1533,12 +1481,12 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG, bases.Statement):
         if six.PY3:
             return None
 
-        if '__metaclass__' in self.locals:
-            assignment = self.locals['__metaclass__'][-1]
+        if '__metaclass__' in self._locals:
+            assignment = self._locals['__metaclass__'][-1]
         elif self.bases:
             return None
-        elif '__metaclass__' in self.root().locals:
-            assignments = [ass for ass in self.root().locals['__metaclass__']
+        elif '__metaclass__' in self.root()._locals:
+            assignments = [ass for ass in self.root()._locals['__metaclass__']
                            if ass.lineno < self.lineno]
             if not assignments:
                 return None
@@ -1559,7 +1507,7 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG, bases.Statement):
             seen = set()
         seen.add(self)
 
-        klass = self.declared_metaclass()
+        klass = self._explicit_metaclass()
         if klass is None:
             for parent in self.ancestors():
                 if parent not in seen:
@@ -1582,7 +1530,7 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG, bases.Statement):
 
     def _islots(self):
         """ Return an iterator with the inferred slots. """
-        if '__slots__' not in self.locals:
+        if '__slots__' not in self._locals:
             return
         for slots in self.igetattr('__slots__'):
             # check if __slots__ is a valid type
@@ -1631,16 +1579,7 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG, bases.Statement):
                 except exceptions.InferenceError:
                     continue
 
-    # Cached, because inferring them all the time is expensive
-    @decorators_mod.cached
-    def slots(self):
-        """Get all the slots for this node.
-
-        If the class doesn't define any slot, through `__slots__`
-        variable, then this function will return a None.
-        Also, it will return None in the case the slots weren't inferred.
-        Otherwise, it will return a list of slot names.
-        """
+    def _slots(self):
         if not self.newstyle:
             raise NotImplementedError(
                 "The concept of slots is undefined for old-style classes.")
@@ -1656,6 +1595,39 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG, bases.Statement):
         # pylint: disable=unsupported-binary-operation; false positive
         return [first] + list(slots)
 
+    # Cached, because inferring them all the time is expensive
+    @decorators_mod.cached
+    def slots(self):
+        """Get all the slots for this node.
+
+        If the class doesn't define any slot, through `__slots__`
+        variable, then this function will return a None.
+        Also, it will return None in the case the slots weren't inferred.
+        Otherwise, it will return a list of slot names.
+        """
+        def grouped_slots():
+            # Not interested in object, since it can't have slots.
+            for cls in self.mro()[:-1]:
+                try:
+                    cls_slots = cls._slots()
+                except NotImplementedError:
+                    continue
+                if cls_slots is not None:
+                    for slot in cls_slots:
+                        yield slot
+                else:
+                    yield None
+
+        if not self.newstyle:
+            raise NotImplementedError(
+                "The concept of slots is undefined for old-style classes.")
+
+        slots = list(grouped_slots())
+        if not all(slot is not None for slot in slots):
+            return None
+
+        return sorted(slots, key=lambda item: item.value)
+
     def _inferred_bases(self, context=None):
         # TODO(cpopa): really similar with .ancestors,
         # but the difference is when one base is inferred,
@@ -1723,9 +1695,13 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG, bases.Statement):
         _verify_duplicates_mro(unmerged_mro)
         return _c3_merge(unmerged_mro)
 
-    def bool_value(self):
-        return True
+def get_locals(node):
+    '''Stub function for forwards compatibility.'''
+    return node._locals
 
+def get_attributes(node):
+    '''Stub function for forwards compatibility.'''
+    return node._instance_attrs
 
 # Backwards-compatibility aliases
 Class = node_classes.proxy_alias('Class', ClassDef)
